# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2019-08-30 07:10
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('xapi', '0003_auto_20190807_1006'),
    ]

    operations = [
        migrations.AlterField(
            model_name='xapilearnerdatatransmissionaudit',
            name='error_message',
            field=models.TextField(blank=True, null=True),
        ),
    ]
