# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-06 16:46
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0021_sapsuccessfactorsenterprisecustomerconfiguration_show_total_hours'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='show_total_hours',
        ),
        migrations.AddField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='transmit_total_hours',
            field=models.BooleanField(default=False, help_text='Include totalHours in the transmitted completion data', verbose_name='Transmit Total Hours'),
        ),
        migrations.AddField(
            model_name='sapsuccessfactorslearnerdatatransmissionaudit',
            name='total_hours',
            field=models.FloatField(blank=True, null=True),
        ),
    ]
