# -*- coding: utf-8 -*-
"""
Integrated channel custom exceptions.
"""
from __future__ import absolute_import, unicode_literals


class ClientError(Exception):
    """
    Indicate a problem when interacting with an integrated channel.
    """
