# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-01-20 11:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0083_enterprisecustomerreportingconfiguration_include_date'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomeruser',
            name='user_id',
            field=models.PositiveIntegerField(db_index=True),
        ),
    ]
