# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-14 20:11
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0053_pendingenrollment_cohort_name'),
        ('enterprise', '0053_auto_20180911_0811'),
    ]

    operations = [
    ]
